"""
Script to validate data form the dataset parser to see if audio and MIDI is valid
"""

import dataset_parsers
import pywav
import mido
from Dataset.sequence_tools import SequenceTools
import note_seq
import wave
import scipy

def validate_note_sequence(ns: note_seq.NoteSequence) -> None:
    """Raise ValueError if NoteSequence contains invalid notes."""
    for note in ns.notes:
        #print(note)
        if note.start_time >= note.end_time:
            raise ValueError('note has start time >= end time: %f >= %f' %
                           (note.start_time, note.end_time))
        if note.velocity == 0:
            raise ValueError('note has zero velocity')

def validate(data_list):
    wav_notok = []
    scipy_notok = []
    midi_notok = []
    midi_invalid = []
    print(data_list)
    for data in data_list:
        print("---------")
        data_id = data["id"]
        audio = data["audio"]
        sequence = data["sequence"]
        print(data_id)
        print(audio)
        print(sequence)
        try:
            f = scipy.io.wavfile.read(audio)
        except:
            scipy_notok.append(data_id)
            print(f"SCIPY {data_id} NOTOK")
        try:
            audio_read = pywav.WavRead(audio)
            print("WAV OK")
        except Exception:
            print("WAV NOTOK")
            wav_notok.append(data_id)

        try:
            sequence_read = mido.MidiFile(sequence)
            print("MIDI OK")
        except Exception:
            print("MIDI NOTOK")
            midi_notok.append(data_id)

        try:
            ns = SequenceTools.midi_to_note_sequence(sequence)
            validate_note_sequence(ns)
        except Exception:
            print("INVALID")
            midi_invalid.append(data_id)

    print("WAV NOTOK: ", wav_notok)
    print("MIDI NOTOK: ", midi_notok)
    print("MIDI INVALID: ", midi_invalid)
    print("SCIPY NOTOK: ", scipy_notok)


if __name__ == '__main__':
    validate(dataset_parsers.parse_musicnet_org('/cluster/home/fredrifw/Datasets/musicnet/tracks/validation/'))