from mido import MidiFile, MidiTrack
import os
import re
from Dataset.sequence_tools import SequenceTools


def midi_split_urmp(urmp_path):
    """
    For each track in urmp:
        Get the midi
        for each track which is not meta
            create a separate midi for the file
            save it as vn/vc/<instrument_tag>_midi.mid
    """
    for track in os.listdir(urmp_path):
        if re.match("^[0-9]{2}_(.*)", track):
            instruments = track.split("_")[2:]
            for track_file in os.listdir(f"{urmp_path}/{track}"):
                if re.match("^[0-9]{2}_(.*)(.mid|.midi)", track_file):
                    org_mid = MidiFile(f"{urmp_path}/{track}/{track_file}")
                    meta = org_mid.tracks[0]
                    for i in range(len(instruments)):
                        new_midi = MidiFile()
                        new_midi.ticks_per_beat = org_mid.ticks_per_beat
                        new_midi.tracks = [meta]
                        new_midi.tracks += [org_mid.tracks[i+1]]
                        for msg in new_midi.tracks[1]:
                            msg.channel = 0
                        new_midi.save(f"{urmp_path}/{track}/{i}_{instruments[i]}.mid")


def midi_split_musicnet(musicnet_path):
    for track in os.listdir(musicnet_path):
        if re.match("^[0-9]{4}$", track):
            for track_file in os.listdir(f"{musicnet_path}/{track}"):
                if re.match("^[0-9]{4}(.*)(.mid)$", track_file):
                    ns = SequenceTools.midi_to_note_sequence(f"{musicnet_path}/{track}/{track_file}")
                    instruments = []
                    for note in ns.notes:
                        if note.program not in instruments:
                            instruments.append(note.program)
                    org_midi = MidiFile(f"{musicnet_path}/{track}/{track_file}")
                    meta = org_midi.tracks[:2]
                    print(f"Splitting track: {track_file}")
                    print(f"Instruments: {instruments}, {len(instruments)}")
                    print(len(org_midi.tracks[:2]))
                    for i in range(len(instruments)):
                        new_midi = MidiFile()
                        new_midi.ticks_per_beat = org_midi.ticks_per_beat
                        new_midi.tracks = meta
                        new_midi.tracks += [org_midi.tracks[i+1]]
                        for msg in new_midi.tracks[2]:
                            try:
                                msg.channel = 0
                            except:
                                pass
                        new_midi.save(f"{musicnet_path}/{track}/{i}_{org_midi.tracks[i+1].name}.mid")


def replace_midi(midi_dir, org_dir):
    for folder in os.listdir(org_dir):
        if re.match("^[0-9].*$", folder):
            for file in os.listdir(f"{org_dir}/{folder}"):
                if file.endswith(".mid") or file.endswith(".midi"):
                    os.remove(f"{org_dir}/{folder}/{file}")
            if os.path.exists(f"{midi_dir}/{folder}.mid"):
                os.rename(f"{midi_dir}/{folder}.mid", f"{org_dir}/{folder}/{folder}.mid")
            else:
                os.rename(f"{midi_dir}/{folder}.midi", f"{org_dir}/{folder}/{folder}.mid")


if __name__ == "__main__":
    #replace_midi("../URMP_GSUTIL_MIDI/train", "../Datasets/URMP/Dataset/train")
    #replace_midi("../URMP_GSUTIL_MIDI/validation", "../Datasets/URMP/Dataset/validation")
    replace_midi("../MUSICNET_MIDI", "../Datasets/musicnet/tracks/train")
    replace_midi("../MUSICNET_MIDI", "../Datasets/musicnet/tracks/validation")
    #midi_split_urmp("../Datasets/URMP/Dataset/train")
    #midi_split_urmp("../Datasets/URMP/Dataset/validation")
    midi_split_musicnet("../Datasets/musicnet/tracks/train")
    midi_split_musicnet("../Datasets/musicnet/tracks/validation")

