from pipeline_generator import PipelineGenerator
import re
import os
import itertools
from random import randint, shuffle
from config import *


class SlakhPipelineGenerator(PipelineGenerator):

    def _loop_rec(self, src_path, name_index, midi_tup, src_root=".", finding_midi=False, _name_key=None, _counter=0):
        """
                Loops the search path recursively and returns a dictionary where a key is any folder
                matching the src_path at the naming_index and the values are all files matching last index in src_path
                :param src_path: Reg-ex search string, directories/files in the search tree are separated by @
                :param name_index: The index in the search string to use as naming keys
                :param midi_tup: A tuple containing of (1): Where to find midi relative to matched files.
                (2): What pattern to use for matching MIDI
                :param src_root: Where to start searching using the src_path
                :param finding_midi: Whether or not the found files are midi
                :param _name_key: To keep track of the naming when iterating recursively - dont supply!
                :param _counter: To keep track of iterations recursively - dont supply!
                :return: None
                """
        """
        For each song you find
            Grab 10 random subsets of min 4 instruments
            
        """
        _counter = _counter + 1
        return_paths = {}
        split_path = src_path.split("@")
        rec_left = len(split_path)
        if rec_left > 1:
            # Find the next folder and iterate
            path = os.walk(src_root)
            for root, directories, _ in path:
                for directory in directories:
                    if _counter == name_index:
                        _name_key = directory
                    if re.match(split_path[0], directory):
                        new_src_root = f"{root}/{directory}"
                        returned_files = self._loop_rec('@'.join(split_path[1:]), name_index, midi_tup, new_src_root,
                                                        finding_midi, _name_key, _counter)
                        return_paths.update(returned_files)
            return return_paths
        else:
            return_paths = {}
            # Voila! You are in the .wav file folder
            bpm = None
            if not finding_midi:
                # Assuming all .wav in this folder follow the same tempo, it suffices to get the tempo of one of the midis
                bpm = self.find_tempo_from_midi_rec(midi_tup, src_root)

            # Make random subsets of min 4 instruments
            combinations = []
            while len(combinations) < 10:
                generated_combo = self.generate_random_combination(os.listdir(src_root))
                if generated_combo.sort() not in combinations:
                    combinations.append(generated_combo.sort())
            # Select 10 subsets at random at return files for these
            for i in range(10):
                combo = combinations[i]
                combo_files = []
                if not bpm:
                    bpm = self.find_tempo_from_midi(f"{src_root}/{combo[0]}")
                for file in combo:
                    combo_files.append(f"{src_root}/{file}")
                return_paths[f"{_name_key}_{'_'.join(combo).replace('.wav', '')}"] = (combo_files, bpm)
            return return_paths

    @staticmethod
    def generate_random_combination(available_stems, min_length=4):
        combination = []
        destionation_length = randint(min_length, len(available_stems))
        while len(combination) < destionation_length:
            shuffle(available_stems)
            combination.append(available_stems.pop())
        return combination


if __name__ == "__main__":
    pg = SlakhPipelineGenerator()
    pg.create_effector_pipeline("Slakh-SSH", SLAKH_LOCAL, SLAKH_LOCAL_NAMING_INDEX, SLAKH_MIDI, [RC20])